require 'common'
require 'encryption'
require 'active_support/core_ext/hash/indifferent_access'

#
# Validation for inputs of config command.
# Author: Nayyara Samuel (nayyara.samuel@opower.com)
#
module MultiIndex
  module ConfigValidation
    include MultiIndex::CommandsCommon
    include MultiIndex::Encryption

    ALLOWED_VALUES = {
        output: ['EAS', 'CONFIG_SERVICE', 'stdout'],
        format: ['yaml', 'json', 'escaped']
    }
    MAPPING_FORMAT = {
        'EAS' => 'yaml',
        'CONFIG_SERVICE' => 'escaped'
    }

    def validate_write_parameters(options)
      output, format = options['output'], options['format']

      #Check preconditions
      check_argument(ALLOWED_VALUES[:output].include?(output), "Output must be one of #{ALLOWED_VALUES[:output]}")

      if (MAPPING_FORMAT[output])
        format = MAPPING_FORMAT[output]
        LOG.info "Format is automatically set to #{format} for #{output}"
      end

      check_argument(ALLOWED_VALUES[:format].include?(format), "Format must be one of #{ALLOWED_VALUES[:format]}")

      {
          output: output,
          format: format
      }
    end

    begin
      require 'io/console'
    rescue LoadError
    end

    if STDIN.respond_to?(:noecho)
      def get_password(prompt="Password: ")
        print prompt
        result = STDIN.noecho(&:gets).chomp
        puts
        result
      end
    else
      def get_password(prompt="Password: ")
        result = `read -s -p "#{prompt}" password; echo $password`.chomp
        puts
        result
      end
    end

    def get_plain(prompt)
      print prompt
      STDIN.gets.chomp
    end

    def validate_config_parameters(options)
      profile = options['profile']
      profile = HashWithIndifferentAccess.new(YAML.load_file(profile)) if profile

      profile ||= {
          google_drive: {},
          ldap: {}
      }

      LOG.info '*** Google Drive Credentials/Access Info ****'
      profile[:google_drive][:email] ||= get_plain('Google Email: ')
      profile[:google_drive][:password] ||= encrypt(get_password('Google password: '))
      profile[:google_drive][:key] ||= get_plain('Document Key: ')
      profile[:google_drive][:sheet] ||= get_plain('Sheet Index: ')

      if (options['output'] == 'CONFIG_SERVICE' && profile[:ldap].empty?)
        LOG.info '*** LDAP Credentials (for Config Service): ****'
        profile[:ldap][:email] ||= get_plain('LDAP Username: ')
        profile[:ldap][:password] ||= encrypt(get_password('LDAP Password: '))
        profile[:ldap][:key] ||= get_plain('Config Key: ')
      end

      options.merge!({profile: profile})
      options
    end
  end
end